//
//  ImmediatePropertyListeners.swift
//  Jetstream
//
//  Copyright (c) 2014 Uber Technologies, Inc.
//
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.

import UIKit
import XCTest

class ImmediatePropertyListenerTests: XCTestCase {
    func testSpecificPropertyListeners() {
        var model = TestModel()
        var dispatchCount = 0
        
        model.observeChange(self, key: "string") {
            dispatchCount += 1
        }
        
        model.string = "test"
        model.string = "test 2"
        model.int = 1
        model.float = 2.5
        
        delayTest(self, 0.01) {
            XCTAssertEqual(dispatchCount, 1 , "Dispatched once")
        }
    }
    
    func testMultiPropertyListeners() {
        var model = TestModel()
        var lastValue: NSString? = ""
        var dispatchCount = 0
        
        model.observeChange(self, keys: ["string", "int"]) {
            dispatchCount += 1
        }
        
        model.string = "test"
        model.int = 1
        model.string = "test"
        model.int = 1
        model.float = 2.5
        
        delayTest(self, 0.01) {
            XCTAssertEqual(dispatchCount, 1 , "Dispatched once")
        }
    }
    
    func testNoDispatchForNoChange() {
        var model = TestModel()
        var lastValue: NSString? = ""
        var dispatchCount = 0
        
        model.observeChange(self) {
            dispatchCount += 1
        }
    
        model.int = 10
        model.int = 10
        
        model.uint = 10
        model.uint = 10
        
        model.uint8 = 10
        model.uint8 = 10
        
        model.int8 = 10
        model.int8 = 10
        
        model.uint16 = 10
        model.uint16 = 10
        
        model.int16 = 10
        model.int16 = 10
        
        model.uint32 = 10
        model.uint32 = 10
  
        model.int32 = 10
        model.int32 = 10

        model.uint64 = 10
        model.uint64 = 10
   
        model.int64 = 10
        model.int64 = 10
   
        model.bool = true
        model.bool = true
 
        model.string = "test"
        model.string = "test"
     
        model.string = "test 2"
        model.string = "test 2"
 
        model.float = 10.0
        model.float = 10.0
 
        model.float = 10.1
        model.float = 10.2
  
        model.double = 10.0
        model.double = 10.0
   
        model.double = 10.1
        model.double = 10.2
    
        model.testType = .Active
        model.testType = .Active
        
        delayTest(self, 0.01) {
            XCTAssertEqual(dispatchCount, 1 , "Dispatched once")
        }
    }
}
